import 'package:cramr_ui_lib/model/conjugation.dart';
import 'package:cramr_ui_lib/model/verb_metadata.dart';
import 'package:flutter/material.dart';

/// Note: the library currently focuses on french. In the future a separate
/// library specifically for french might be created next to a more generic one
/// Represents a verb. It consists of the [infinitive], the [translation] in the
/// language of the client app, the complete conjugation of the
/// [simplePresentIndicative] (present), the [simplePastIndicative] and the
/// language ([isoLanguage]) of the verb that is being practiced
class Verb with ChangeNotifier {
  late final VerbMetadata _metadata;
  final Map<String, Conjugation> _indicativeConjugations = {};

  List<String>? get simplePastIndicative =>
      _indicativeConjugations['indicatif_imparfait']?.conjugation;

  String get isoLanguage => _metadata.isoLanguage;

  String get infinitive => _metadata.infinitive;

  String get translation => _metadata.translation;

  List<String> get simplePresentIndicative =>
      _indicativeConjugations['indicatif_présent']!.conjugation;

  List<String> get simpleFutureIndicative =>
      _indicativeConjugations['indicatif_futur simple']!.conjugation;

  set simplePresentIndicative(List<String> indicative) {
    assert(indicative.isNotEmpty, 'indicative cannot be empty');
    assert(indicative.length == 6, 'indicative should contain six elements');
    Conjugation _conjugation = Conjugation('indicatif', 'présent', indicative);
    _indicativeConjugations['présent'] = _conjugation;
    notifyListeners();
  }

  set simplePastIndicative(List<String>? indicative) {
    assert(indicative!.isNotEmpty, 'indicative cannot be empty');
    assert(indicative!.length == 6, 'indicative should contain six elements');
    Conjugation _conjugation =
        Conjugation('indicatif', 'imparfait', indicative!);
    _indicativeConjugations['imparfait'] = _conjugation;
    notifyListeners();
  }

  Map<String, Conjugation> get indicativeConjugations =>
      _indicativeConjugations;

  bool hasSimplePastIndicative() => simplePastIndicative?.isNotEmpty ?? false;

  Conjugation? conjugation(String mode, String tense) {
    for (var element in indicativeConjugations.entries) {
      if (mode == element.value.mode && tense == element.value.tense) {
        return element.value;
      }
    }
    return null;
  }

  Verb(String infinitive, String translation,
      Conjugation simplePresentIndicative, String isoLanguage) {
    _metadata = VerbMetadata(infinitive, translation, isoLanguage);
    _indicativeConjugations['indicatif_présent'] = simplePresentIndicative;
  }

  Verb.fromMoreTensesAndModes(
      String infinitive,
      String translation,
      Conjugation simplePresentIndicative,
      Conjugation simplePastIndicative,
      String isoLanguage) {
    _metadata = VerbMetadata(infinitive, translation, isoLanguage);
    _indicativeConjugations['indicatif_présent'] = simplePresentIndicative;
    _indicativeConjugations['indicatif_imparfait'] = simplePastIndicative;
  }

  Verb.fromAllTensesAndModes(String infinitive, String translation,
      List<Conjugation> conjugations, String isoLanguage) {
    _metadata = VerbMetadata(infinitive, translation, isoLanguage);
    for (var element in conjugations) {
      _indicativeConjugations[element.getIdentifier()] = element;
    }
  }
}
