import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/routes/verb_selection_route.dart';
import 'package:cramr_ui_lib/routes/verb_studymode_selection_route.dart';
import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/settings/settings_view.dart';
import 'package:cramr_ui_lib/widgets/score_widget.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import '../widgets/conjugation_meta_data_card.dart';
import '../widgets/evalutation_row.dart';

/// The Route that allows you to show the evaluation of the indicative of the
/// [Verb] that you are studying.
class VerbIndicativeEvaluateRoute extends StatelessWidget {
  static const routeName = '/verbEvaluateRoute';

  final SettingsController _settingsController;

  const VerbIndicativeEvaluateRoute(this._settingsController, {Key? key})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
      animation: _settingsController,
      builder: (BuildContext context, Widget? child) {
        return MaterialApp(
          debugShowCheckedModeBanner: false,
          themeMode: _settingsController.themeMode,
          theme: ThemeData(),
          darkTheme: ThemeData.dark(),
          home: SafeArea(
            child: Scaffold(
              appBar: AppBar(
                actions: [
                  _repeatIcon(context),
                  _homeIcon(context),
                  _settingsIcon(context)
                ],
                title: _titleCell(context),
              ),
              body: _centerWidget(context),
            ),
          ),
        );
      },
    );
  }

  IconButton _settingsIcon(BuildContext context) {
    return IconButton(
        key: const Key('settingsButton'),
        icon: const Icon(Icons.settings),
        onPressed: () {
          Navigator.restorablePushNamed(context, SettingsView.routeName);
        });
  }

  IconButton _homeIcon(BuildContext context) {
    final VerbTestModel testModel =
        Provider.of<VerbTestModel>(context, listen: false);
    return IconButton(
        key: const Key('menu_key'),
        onPressed: () {
          testModel.reset();
          Navigator.restorablePopAndPushNamed(
              context, VerbSelectionRoute.routeName);
        },
        icon: const Icon(Icons.home));
  }

  IconButton _repeatIcon(BuildContext context) {
    final VerbTestModel testModel =
        Provider.of<VerbTestModel>(context, listen: false);
    return IconButton(
        key: const Key('repeat_key'),
        onPressed: () {
          testModel.reset();
          Navigator.popAndPushNamed(
              context, VerbStudymodeSelectionRoute.routeName);
        },
        icon: const Icon(Icons.refresh));
  }

  Wrap _titleCell(BuildContext context) {
    return Wrap(direction: Axis.horizontal, children: const [
      Text(
        "Je score",
        textAlign: TextAlign.center,
      ),
    ]);
  }

  Widget _centerWidget(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        key: UniqueKey(),
        body: _bodyListView(context),
      ),
    );
  }

  Widget _scoreWidget(BuildContext context) {
    VerbTestModel model = context.watch<VerbTestModel>();
    return ScoreWidget(model.score, model.items.length);
  }

  Widget _resultCell(BuildContext context, person) {
    VerbTestModel model = context.watch<VerbTestModel>();
    String actual = model.items[person];
    String expected = model.expected[person];
    return EvaluationRow(person, model.targetLanguageIso,
        model.pronouns[person], actual, expected);
  }

  Widget _bodyListView(BuildContext context) {
    return Column(
      children: [
        _conjugationMetaDataCard(context),
        const Divider(),
        Expanded(
          child: ListView(
            key: UniqueKey(),
            scrollDirection: Axis.vertical,
            children: _bodyListTiles(context),
          ),
        ),
      ],
    );
  }

  Widget _conjugationMetaDataCard(BuildContext context) {
    return Consumer<VerbTestModel>(builder: (context, testModel, child) {
      return ConjugationMetaDataCard(
          infinitive: testModel.infinitive,
          translation: testModel.translation,
          mode: testModel.mode,
          tense: testModel.tense);
    });
  }

  List<Widget> _bodyListTiles(BuildContext context) {
    List<Widget> _widgets = [];

    _widgets.add(_scoreWidget(context));
    _widgets.add(const Divider());

    List<Widget> _rows = [];
    for (int i = 0; i < _itemCount(context); i++) {
      _rows.add(Card(
        key: UniqueKey(),
        elevation: 10,
        child: Padding(
          key: UniqueKey(),
          padding: const EdgeInsets.all(5.0),
          child: _resultCell(context, i),
        ),
      ));
    }

    _widgets.add(Padding(
      key: UniqueKey(),
      padding: const EdgeInsets.fromLTRB(20, 0, 20, 0),
      child: Card(
        key: UniqueKey(),
        child: Column(
          key: UniqueKey(),
          children: _rows,
        ),
      ),
    ));

    return _widgets;
  }

  int _itemCount(BuildContext context) {
    return Provider.of<VerbTestModel>(context, listen: false).items.length;
  }
}
