import 'dart:collection';

import 'package:cramr_ui_lib/model/cardinality_selection.dart';
import 'package:cramr_ui_lib/model/verb.dart';
import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/repo/verb_repository.dart';
import 'package:cramr_ui_lib/routes/verb_studymode_selection_route.dart';
import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/settings/settings_view.dart';
import 'package:cramr_ui_lib/theme/custom_theme.dart';
import 'package:cramr_ui_lib/util/cardinality_filter.dart';
import 'package:cramr_ui_lib/util/text_size_limiter.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import '../widgets/verb_card.dart';

/// The Route that allows you to select a verb for evaluation
class VerbSelectionRoute extends StatefulWidget {
  static const routeName = '/verbSelectionRoute';

  final SettingsController _settingsController;

  const VerbSelectionRoute(this._settingsController, {Key? key})
      : super(key: key);

  @override
  State<VerbSelectionRoute> createState() => VerbSelectionRouteState();

  static Key verbIconKey(String infinitive, String mode, String tense) {
    return Key('startStudy_${infinitive}_${mode}_$tense');
  }
}

class VerbSelectionRouteState extends State<VerbSelectionRoute> {
  final HashMap<String, Verb> verbs = VerbRepository.instance().verbs;

  List<String>? filteredVerbs = [];
  bool isByVerbList = true;
  String verbListMode = 'werkwoord';

  late List<VerbPanelItem> _verbPanelItems;

  HashMap<String, List<String>> _tensesPerMode() {
    //two iterations over the verbs:
    //first to make sure we have a set of all Conjugations available
    //secondly we append the verbs to the conjugations
    HashMap<String, List<String>> conjugations = HashMap();
    for (var verb in verbs.values) {
      for (var conjugation in verb.indicativeConjugations.values) {
        if (!conjugations.containsKey(conjugation.mode)) {
          conjugations[conjugation.mode] = [conjugation.tense];
        } else {
          // mode is present, check for conjugation
          List<String>? modeConjugations = conjugations[conjugation.mode];
          if (null != modeConjugations) {
            if (!modeConjugations.contains(conjugation.tense)) {
              modeConjugations.add(conjugation.tense);
            }
          }
        }
      }
    }
    return conjugations;
  }

  List<VerbPanelItem> generateItems() {
    HashMap<String, List<String>> conjugations = _tensesPerMode();
    List<VerbPanelItem> panelItems = [];
    for (String mode in conjugations.keys) {
      var conjugation = conjugations[mode];
      if (conjugation != null) {
        for (String tense in conjugation) {
          panelItems.add(VerbPanelItem(mode: mode, tense: tense));
        }
      }
    }
    return panelItems;
  }

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
        animation: widget._settingsController,
        builder: (BuildContext context, Widget? child) {
          return MaterialApp(
            debugShowCheckedModeBanner: false,
            themeMode: widget._settingsController.themeMode,
            theme: ThemeData(),
            darkTheme: ThemeData.dark(),
            home: SafeArea(
              child: Scaffold(
                appBar: AppBar(
                  title: _listModeDropdown(context),
                  actions: [_searchIcon(context), _settingsIcon(context)],
                ),
                body: Center(
                  key: const Key('verbSelectionCenter'),
                  child: _verbListView(context),
                ),
                // This trailing comma makes auto-formatting nicer for build methods.
              ),
            ),
          );
        });
  }

  Widget _verbListView(BuildContext context) {
    if (isByVerbList) {
      return ListView(
        key: const Key('verbListView'),
        scrollDirection: Axis.vertical,
        controller: ScrollController(),
        physics: const ScrollPhysics(
          parent: AlwaysScrollableScrollPhysics(),
        ),
        children: _repoButtons(context),
      );
    } else {
      return SingleChildScrollView(
        child: _conjugationList(context),
      );
    }
  }

  Widget _conjugationList(BuildContext context) {
    return ExpansionPanelList(
      expansionCallback: (int index, bool isExpanded) {
        setState(() {
          _verbPanelItems[index].isExpanded = !isExpanded;
        });
      },
      children: _verbPanelItems.map<ExpansionPanel>((VerbPanelItem item) {
        return ExpansionPanel(
            headerBuilder: (BuildContext context, bool isExpanded) {
              String _mode = item.mode;
              String _tense = item.tense;
              return ListTile(
                title: SizedBox(
                  key: UniqueKey(),
                  height: 40,
                  child: Text(
                    '$_mode $_tense',
                    key: Key('vervoeg_${_mode}_$_tense'),
                    style: CustomTheme.lightTheme.textTheme.headline1,
                    textScaleFactor: TextSizeLimiter.textScaleFactor(context),
                  ),
                ),
              );
            },
            body: ListTile(
              title: _conjugationButton(context, item.mode, item.tense),
            ),
            isExpanded: item.isExpanded);
      }).toList(),
    );
  }

  IconButton _searchIcon(BuildContext context) {
    return IconButton(
      key: const Key("searchVerbButton"),
      onPressed: () async {
        String? filteredVerb = await showSearch(
            context: context,
            delegate: VerbSearchDelegate(),
            useRootNavigator: true);
        setState(() {
          filteredVerbs!.clear();
          filteredVerbs!.add(filteredVerb!);
        });
      },
      icon: const Icon(Icons.search),
    );
  }

  IconButton _settingsIcon(BuildContext context) {
    return IconButton(
        key: const Key('settingsButton'),
        icon: const Icon(Icons.settings),
        onPressed: () {
          Navigator.pushNamed(context, SettingsView.routeName);
        });
  }

  DropdownButton<String> _listModeDropdown(BuildContext context) {
    return DropdownButton<String>(
        value: verbListMode,
        items: <String>['werkwoord', 'vervoeging']
            .map<DropdownMenuItem<String>>((String value) {
          return DropdownMenuItem<String>(
            child: Text(value),
            value: value,
          );
        }).toList(),
        onChanged: (String? selection) {
          setState(() {
            verbListMode = selection!;
            isByVerbList = !isByVerbList;
          });
        });
  }

  List<Widget> _repoButtons(BuildContext context) {
    List<Widget> result = [];

    if (verbListMode == 'werkwoord') {
      for (var key in filteredVerbs!) {
        if (verbs.containsKey(key)) {
          Verb? value = verbs[key];
          result.add(SizedBox(
            child: Card(
              key: Key('verbCard_$value'),
              child: _verbButton(context, value!),
              margin: const EdgeInsets.fromLTRB(30, 15, 30, 15),
              elevation: 10.0,
            ),
          ));
        }
      }
    }

    return result;
  }

  Widget _conjugationButton(BuildContext context, String mode, String tense) {
    return Padding(
        padding: const EdgeInsets.all(8.0),
        child: Column(
          children: [
            Card(
              elevation: 5.0,
              child: Column(children: _verbCards(context, mode, tense)),
            )
          ],
        ));
  }

  List<Widget> _verbCards(BuildContext context, String mode, String tense) {
    List<Widget> result = [];
    for (var key in filteredVerbs!) {
      if (null != verbs[key]) {
        Verb? verb = verbs[key];
        result.add(_verbCard(context, mode, tense, verb!));
      }
    }
    return result;
  }

  Widget _verbButton(BuildContext context, Verb verb) {
    return Padding(
      padding: const EdgeInsets.all(8.0),
      child: Column(
        key: Key("vervoeg ${verb.infinitive}"),
        children: [
          SizedBox(
            key: UniqueKey(),
            height: 40,
            child: Text(
              verb.infinitive,
              key: UniqueKey(),
              style: CustomTheme.lightTheme.textTheme.headline1,
              textScaleFactor: TextSizeLimiter.textScaleFactor(context),
            ),
          ),
          SizedBox(
            key: UniqueKey(),
            height: 40,
            child: Text(
              verb.translation,
              key: UniqueKey(),
              style: CustomTheme.lightTheme.textTheme.bodyText2,
              textScaleFactor: TextSizeLimiter.textScaleFactor(context),
            ),
          ),
          verb.hasSimplePastIndicative()
              ? Card(
                  elevation: 5.0,
                  child: Column(children: _conjugationCards(context, verb)),
                )
              : _verbIcon(context, verb, null, null),
        ],
      ),
    );
  }

  List<Widget> _conjugationCards(BuildContext context, Verb verb) {
    List<Widget> result = [];
    verb.indicativeConjugations.forEach((key, value) {
      result.add(_conjugationCard(context, value.mode, value.tense, verb));
    });
    return result;
  }

  Widget _conjugationCard(
      BuildContext context, String mode, String tense, Verb verb) {
    return Card(
      elevation: 5.0,
      child: ListTile(
        title: Text(
          '$mode $tense',
          style: CustomTheme.lightTheme.textTheme.caption,
        ),
        trailing: _verbIcon(context, verb, mode, tense),
      ),
    );
  }

  Widget _verbCard(BuildContext context, String mode, String tense, Verb verb) {
    return VerbCard(
        infinitive: verb.infinitive,
        verbIcon: _verbIcon(context, verb, mode, tense));
  }

  Widget _verbIcon(
      BuildContext context, Verb verb, String? mode, String? tense) {
    Key iconKey = Key('startStudy_${verb.infinitive}');
    List<String>? indicative = verb.simplePresentIndicative;
    if (null != tense) {
      if ('présent' == tense && 'indicatif' == mode) {
        iconKey = Key('startStudy_${verb.infinitive}_indicatif_présent');
        indicative = verb.simplePresentIndicative;
      } else {
        iconKey = VerbSelectionRoute.verbIconKey(verb.infinitive, mode!, tense);
        indicative =
            verb.indicativeConjugations[mode + '_' + tense]!.conjugation;
      }
    }
    return InkWell(
      key: UniqueKey(),
      child: CircleAvatar(
          key: UniqueKey(), child: Icon(Icons.play_arrow, key: iconKey)),
      onTap: () {
        final VerbTestModel model =
            Provider.of<VerbTestModel>(context, listen: false);
        final CardinalitySelection selection =
            widget._settingsController.cardinalitySelection;
        model.initialise(
            CardinalityFilter.filteredList(selection, indicative!),
            CardinalityFilter.filteredList(
                selection, VerbRepository.instance().pronouns),
            verb.infinitive,
            verb.isoLanguage,
            verb.translation,
            mode!,
            tense!);
        model.reset();
        Navigator.restorablePopAndPushNamed(
            context, VerbStudymodeSelectionRoute.routeName);
      },
    );
  }

  @override
  void initState() {
    super.initState();

    verbs.forEach((key, value) {
      filteredVerbs!.add(key);
    });

    _verbPanelItems = generateItems();
  }
}

class VerbSearchDelegate extends SearchDelegate<String> {
  List<String> allVerbs = [];
  List<String> suggestedVerbs = [];

  VerbSearchDelegate() {
    var verbs = VerbRepository.instance().verbs;
    for (var element in verbs.keys) {
      allVerbs.add(element);
    }
  }

  @override
  List<Widget>? buildActions(BuildContext context) {
    List<Widget> actions = [];
    return actions;
  }

  @override
  Widget? buildLeading(BuildContext context) {
    return null;
  }

  @override
  Widget buildResults(BuildContext context) {
    for (var element in allVerbs) {
      if (element.toLowerCase().contains(query.toLowerCase())) {
        suggestedVerbs.add(element);
      }
    }

    return ListView.builder(
        itemCount: suggestedVerbs.length,
        itemBuilder: (BuildContext context, int index) {
          var result = suggestedVerbs[index];
          return ListTile(
            title: Text(
              result,
              key: Key("searchResult_$result"),
              textScaleFactor: TextSizeLimiter.textScaleFactor(context),
            ),
            onTap: () {
              close(context, result);
            },
          );
        });
  }

  @override
  Widget buildSuggestions(BuildContext context) {
    suggestedVerbs.clear();
    for (var element in allVerbs) {
      if (element.toLowerCase().contains(query.toLowerCase())) {
        suggestedVerbs.add(element);
      }
    }

    return ListView.builder(
        itemCount: suggestedVerbs.length,
        itemBuilder: (BuildContext context, int index) {
          var result = suggestedVerbs[index];
          return ListTile(
            title: Text(
              result,
              key: Key("suggestedResult_$result"),
              textScaleFactor: TextSizeLimiter.textScaleFactor(context),
            ),
            onTap: () {
              close(context, result);
            },
          );
        });
  }
}

class VerbPanelItem {
  late final String mode;
  late final String tense;
  bool isExpanded;

  VerbPanelItem(
      {required this.mode, required this.tense, this.isExpanded = false});
}
